/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.file.storage;

import java.util.EnumSet;
import com.openexchange.capabilities.CapabilityService;
import com.openexchange.capabilities.CapabilitySet;
import com.openexchange.exception.OXException;
import com.openexchange.file.storage.composition.DelegatingIDBasedFileAccess;
import com.openexchange.file.storage.composition.IDBasedFileAccess;
import com.openexchange.file.storage.composition.crypto.CryptographicAwareIDBasedFileAccessFactory;
import com.openexchange.file.storage.composition.crypto.CryptographyMode;
import com.openexchange.guard.exceptions.GuardExceptionCodes;
import com.openexchange.guard.internal.AbstractGuardAccess;
import com.openexchange.guard.internal.authentication.GuardAuthenticationToken;
import com.openexchange.session.Session;

/**
 * {@link GuardAwareIDBasedFileAccessFactoryImpl}
 *
 * @author <a href="mailto:benjamin.gruedelbach@open-xchange.com">Benjamin Gruedelbach</a>
 * @since v7.8.2
 */
public class GuardAwareIDBasedFileAccessFactoryImpl extends AbstractGuardAccess implements CryptographicAwareIDBasedFileAccessFactory {

    private final EncryptedFileRecognizer[] encryptedFileRecognizers = new EncryptedFileRecognizer[] { new MetadataFileRecognizer(), new NameBasedEncryptedFileRecognizer(), new ContentTypeFileRecognizer() };
    private final CapabilityService capabilityService;
    private final static String GUARD_DRIVE_CAP = "guard-drive";

    /**
     * Initializes a new {@link GuardAwareIDBasedFileAccessFactoryImpl}.
     *
     * @param capabilityService The {@link CapabilityService}
     */
    public GuardAwareIDBasedFileAccessFactoryImpl(CapabilityService capabilityService) {
        this.capabilityService = capabilityService;
    }

    /**
     * Internal method to construct an {@link GuardAuthenticationToken} from a given string.
     *
     * @param authentication The string representation of the token.
     * @return The {@link GuardAuthenticationToken} created from the given string
     * @throws OXException
     */
    private GuardAuthenticationToken createToken(String authentication) throws OXException {
        try {
            if(authentication != null) {
                GuardAuthenticationToken token = GuardAuthenticationToken.fromString(authentication);
                if (token != null) {
                    return token;
                } else {
                    throw GuardExceptionCodes.MISSING_AUTH.create();
                }
            } else {
                //Authentication is not required for all crypto modes
                return null;
            }
        } catch (Exception e) {
            throw GuardExceptionCodes.MISSING_AUTH.create();
        }
    }

    /**
     * Checks if the given session has guard-drive capability set.
     *
     * @param session The session
     * @return True if the session has the guard-drive capability, false otherwise
     * @throws OXException
     */
    private boolean hasGuardDriveCapabilities(Session session) throws OXException {
        CapabilitySet capabilities = this.capabilityService.getCapabilities(session);
        return capabilities.contains(GUARD_DRIVE_CAP);
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.guard.api.filestorage.GuardAwareIDBasedFileAccessFactory#createAccess(com.openexchange.file.storage.composition.IDBasedFileAccess, com.openexchange.session.Session,
     * com.openexchange.guard.api.authentication.GuardAuthenticationToken)
     */
    @Override
    public IDBasedFileAccess createAccess(IDBasedFileAccess fileAccess, EnumSet<CryptographyMode> modes, Session session, String authentication) throws OXException {
        if(hasGuardDriveCapabilities(session)) {
            return new GuardAwareIDBasedFileAccess(fileAccess,
                modes,
                requireGuardApi(PGPCORE_ENDPOINT),
                session,
                new CompositeEncryptedFileRecognizer(encryptedFileRecognizers),
                createToken(authentication));
        } else {
            throw OXException.noPermissionForModule(GUARD_DRIVE_CAP);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.file.storage.composition.DelegatingIDBasedFileAccessFactory#createAccess(com.openexchange.file.storage.composition.IDBasedFileAccess, com.openexchange.session.Session)
     */
    @Override
    public DelegatingIDBasedFileAccess createAccess(IDBasedFileAccess fileAccess, EnumSet<CryptographyMode> modes, Session session) throws OXException {
        if(hasGuardDriveCapabilities(session)) {
            return new GuardAwareIDBasedFileAccess(fileAccess,
                modes,
                requireGuardApi(PGPCORE_ENDPOINT),
                session,
                new CompositeEncryptedFileRecognizer(encryptedFileRecognizers));
        } else {
            throw OXException.noPermissionForModule(GUARD_DRIVE_CAP);
        }
    }
}
