/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.json.authentication;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import org.json.JSONException;
import org.json.JSONObject;
import com.openexchange.ajax.requesthandler.AJAXRequestData;
import com.openexchange.ajax.requesthandler.crypto.CryptographicServiceAuthenticationFactory;
import com.openexchange.exception.OXException;
import com.openexchange.guard.internal.authentication.AuthenticationTokenHandler;
import com.openexchange.guard.internal.authentication.GuardAuthenticationToken;
import com.openexchange.session.Session;
import com.openexchange.tools.servlet.AjaxExceptionCodes;

/**
 * {@link GuardAuthenticationFactory} creates OX Guard specific authentication tokens from given HTTP requests.
 *
 * @author <a href="mailto:benjamin.gruedelbach@open-xchange.com">Benjamin Gruedelbach</a>
 * @since v7.8.3
 */
public class GuardAuthenticationFactory implements CryptographicServiceAuthenticationFactory {

    public static final String VALUE_DELIMITER = ":";
    private static final String AUTH_TOKEN_PARAMETER_NAME = "cryptoAuth";
    private static final String GUARD_AUTH_COOKIE_NAME = "cryptoAuth";
    private static final String GUARD_SESSION_COOKIE_NAME = "JSESSIONID";
    private static final String JSON_PARAMETER = "json_0";
    private static final String SECURITY_JSON = "security";
    private static final String AUTHENTICATION_JSON = "authentication";

    /**
     * Internal method to obtain a certain cookie value from a list of cookies.
     *
     * @param cookies The list of cookies.
     * @param cookieName The name of the cookie to obtain the value from.
     * @return The value of the cookie with the given name, or null if no such cookie is present.
     */
    private String getValueFromCookie(Cookie[] cookies, String cookieName) {
        String result = null;
        if (cookies != null) {
            for (final Cookie cookie : cookies) {
                if (cookie.getName().equals(cookieName)) {
                    result = cookie.getValue();
                }
            }
        }
        return result;
    }

    /**
     * Internal method to construct the authentication string used for accessing OX Guard.
     *
     * @param sessionValue The OX Guard Session value.
     * @param authenticationToken The OX Guard authentication token
     * @return The authentication string which can be used for accessing OX Guard.
     */
    private String buildAuthentication(String sessionValue, String authenticationToken) {
        if (sessionValue != null && authenticationToken != null && !authenticationToken.equals("null")) {
            return sessionValue + VALUE_DELIMITER + authenticationToken;
        }
        return null;
    }

    @Override
    public String getTokenValueFromString (String string) {
        GuardAuthenticationToken token = GuardAuthenticationToken.fromString(string);
        if (token == null) return null;
        try {
            return URLEncoder.encode(token.getValue(), "UTF-8");
        } catch (UnsupportedEncodingException e) {
            return null;
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.file.storage.json.crypto.EncryptionServiceAuthenticationFactory#createAuthenticationFrom(javax.servlet.http.HttpServletRequest)
     */
    @Override
    public String createAuthenticationFrom(HttpServletRequest request) {

        if (request == null) {
            return null;
        }

        final String sessionValue = getValueFromCookie(request.getCookies(), GUARD_SESSION_COOKIE_NAME);
        String authToken = getValueFromCookie(request.getCookies(), GUARD_AUTH_COOKIE_NAME);
        if (authToken == null) {
            authToken = request.getParameter(AUTH_TOKEN_PARAMETER_NAME);
        }

        return buildAuthentication(sessionValue, authToken);
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.ajax.requesthandler.crypto.CryptographicServiceAuthenticationFactory#createAuthenticationFrom(javax.servlet.http.HttpServletRequest, com.openexchange.ajax.requesthandler.AJAXRequestData)
     */
    @Override
    public String createAuthenticationFrom(AJAXRequestData requestData) throws OXException {

        if (requestData == null) {
            return null;
        }

        //Fetching authentication from the request object if possible
        final HttpServletRequest request = requestData.optHttpServletRequest();
        if(request != null) {
            String authFromRequest = createAuthenticationFrom(request);
            if(authFromRequest != null) {
                return authFromRequest;
            }
        }

        //Fetching authentication from JSON data if available
        try {
            if(requestData.getUploadEvent() != null) {
                final String formField = requestData.getUploadEvent().getFormField(JSON_PARAMETER);
                if (formField != null) {
                    final JSONObject jsonObject = new JSONObject(formField);
                    if (jsonObject.has(SECURITY_JSON)) {
                        return buildFromJSONObject (
                            jsonObject,
                            getValueFromCookie(request.getCookies(), GUARD_SESSION_COOKIE_NAME));

                    }
                    return null;
                }
            } else if (requestData.getData() instanceof JSONObject) {
                return buildFromJSONObject (
                    (JSONObject) requestData.getData(),
                    getValueFromCookie(request.getCookies(), GUARD_SESSION_COOKIE_NAME));
            }
        } catch (JSONException e) {
            throw AjaxExceptionCodes.JSON_ERROR.create(e, e.getMessage());
        }
        return null;
    }

    private String buildFromJSONObject (JSONObject jsonObject, String sessionValue) throws JSONException {
        if (jsonObject.has(SECURITY_JSON)) {
            String authToken = null;
            final JSONObject jsonSecurity = jsonObject.getJSONObject(SECURITY_JSON);
            if (jsonSecurity.has(AUTHENTICATION_JSON)) {
                authToken = jsonSecurity.getString(AUTHENTICATION_JSON);
            }

            return buildAuthentication(sessionValue, authToken);
        }
        return null;
    }

    /* (non-Javadoc)
     * Non destructively retrieves authentication token from session
     */
    @Override
    public String getAuthTokenFromSession(Session session) throws OXException {
        GuardAuthenticationToken token = new AuthenticationTokenHandler().getForSession(session, false);
        if (token == null) return null;
        return token.getValue();
    }

}
