/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.mail.service.impl;

import com.openexchange.exception.OXException;
import com.openexchange.guard.exceptions.GuardExceptionCodes;
import com.openexchange.guard.internal.authentication.GuardAuthenticationToken;
import com.openexchange.mail.api.IMailFolderStorage;
import com.openexchange.mail.api.IMailMessageStorage;
import com.openexchange.mail.api.MailAccess;
import com.openexchange.mail.api.crypto.CryptographicAwareMailAccessFactory;
import com.openexchange.mail.mime.crypto.PGPMailRecognizer;
import com.openexchange.session.Session;

/**
 * {@link GuardAwareMailAccessFactory}
 *
 * @author <a href="mailto:benjamin.gruedelbach@open-xchange.com">Benjamin Gruedelbach</a>
 * @since v7.8.4
 */
public class GuardAwareMailAccessFactory implements CryptographicAwareMailAccessFactory {

    private static final GuardAuthenticationToken NULL_TOKEN = null;
    private final PGPMailRecognizer pgpMailRecognizer;

    /**
     * Initializes a new {@link GuardAwareMailAccessFactory}.
     * @param pgpMailRecognizer Strategy to use for recognizing PGP email
     */
    public GuardAwareMailAccessFactory(PGPMailRecognizer pgpMailRecognizer) {
        this.pgpMailRecognizer = pgpMailRecognizer;
    }

    /**
     * Internal method to construct an {@link GuardAuthenticationToken} from a given string.
     *
     * @param authentication The string representation of the token.
     * @return The {@link GuardAuthenticationToken} created from the given string
     * @throws OXException
     */
    private GuardAuthenticationToken createToken(String authentication) throws OXException {
        try {
            if(authentication != null) {
                GuardAuthenticationToken token = GuardAuthenticationToken.fromString(authentication);
                if (token != null) {
                    return token;
                } else {
                    throw GuardExceptionCodes.MISSING_AUTH.create();
                }
            } else {
                //Authentication is not required for all crypto modes
                return null;
            }
        } catch (Exception e) {
            throw GuardExceptionCodes.MISSING_AUTH.create();
        }
    }

    /* (non-Javadoc)
     * @see com.openexchange.mail.api.crypto.CryptographicAwareMailAccessFactory#createAccess(com.openexchange.mail.api.MailAccess, com.openexchange.session.Session, java.lang.String)
     */
    @Override
    public MailAccess<IMailFolderStorage, IMailMessageStorage> createAccess(MailAccess<IMailFolderStorage, IMailMessageStorage> delegate, Session session, String authentication) throws OXException {
        return new GuardMailAccess(delegate, session, pgpMailRecognizer, createToken(authentication));
    }

    /* (non-Javadoc)
     * @see com.openexchange.mail.api.crypto.CryptographicAwareMailAccessFactory#createAccess(com.openexchange.mail.api.MailAccess, com.openexchange.session.Session)
     */
    @Override
    public MailAccess<IMailFolderStorage, IMailMessageStorage> createAccess(MailAccess<IMailFolderStorage, IMailMessageStorage> delegate, Session session) throws OXException {
        return new GuardMailAccess(delegate, session, pgpMailRecognizer, NULL_TOKEN);
    }

}
