/*
 *
 *    OPEN-XCHANGE legal inforessagemation
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.mail.service.impl;

import com.openexchange.exception.OXException;
import com.openexchange.guard.exceptions.GuardExceptionCodes;
import com.openexchange.guard.internal.authentication.AuthenticationTokenHandler;
import com.openexchange.guard.internal.authentication.GuardAuthenticationToken;
import com.openexchange.mail.IndexRange;
import com.openexchange.mail.MailField;
import com.openexchange.mail.MailFields;
import com.openexchange.mail.MailSortField;
import com.openexchange.mail.OrderDirection;
import com.openexchange.mail.api.IMailMessageStorage;
import com.openexchange.mail.api.IMailMessageStorageDelegator;
import com.openexchange.mail.api.IMailMessageStorageExt;
import com.openexchange.mail.api.IMailMessageStorageMimeSupport;
import com.openexchange.mail.api.IMailMessageStorageThreadReferences;
import com.openexchange.mail.api.ISimplifiedThreadStructure;
import com.openexchange.mail.api.ISimplifiedThreadStructureEnhanced;
import com.openexchange.mail.api.MailMessageStorage;
import com.openexchange.mail.dataobjects.MailMessage;
import com.openexchange.mail.dataobjects.compose.ComposedMailMessage;
import com.openexchange.mail.mime.crypto.PGPMailRecognizer;
import com.openexchange.mail.search.SearchTerm;
import com.openexchange.session.Session;

/**
 * {@link GuardMessageStorage}
 *
 * @author <a href="mailto:benjamin.gruedelbach@open-xchange.com">Benjamin Gruedelbach</a>
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 * @since v7.8.4
 */
public class GuardMessageStorage extends MailMessageStorage implements IMailMessageStorageDelegator {

    private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(GuardMessageStorage.class);

    protected final IMailMessageStorage delegate;
    protected final Session session;
    protected final PGPMailRecognizer pgpMailRecognizer;
    private final GuardAuthenticationToken authToken;

    /**
     * Initializes a new {@link GuardMessageStorage}.
     *
     * @param session A {@link Session} a session object
     * @param delegate The underlying {@link IMailMessageStorage} to wrap with guard.
     * @param pgpMailRecognizer A {@link PGPMailRecognizer} object recognizes PGP messages.
     */
    public GuardMessageStorage(Session session, IMailMessageStorage delegate, PGPMailRecognizer pgpMailRecognizer, GuardAuthenticationToken authToken) {
        this.session = session;
        this.delegate = delegate;
        this.pgpMailRecognizer = pgpMailRecognizer;
        this.authToken = authToken;
    }

    @Override
    public <T> T supports(Class<T> iface) throws OXException {
        T instance = super.supports(iface);
        if (null == instance) {
            return null;
        }

        if (IMailMessageStorageExt.class == iface) {
            return (T) new GuardMessageStorageExt(session, (IMailMessageStorageExt) instance, pgpMailRecognizer, authToken);
        }

        if (IMailMessageStorageMimeSupport.class == iface) {
            return (T) new GuardMessageStorageMimeSupport(session, (IMailMessageStorageMimeSupport) instance, pgpMailRecognizer, authToken);
        }

        if (IMailMessageStorageThreadReferences.class == iface) {
            return (T) new GuardMessageStorageThreadReferences(session, (IMailMessageStorageThreadReferences) instance, pgpMailRecognizer, authToken);
        }

        if (ISimplifiedThreadStructureEnhanced.class == iface) {
            return (T) new GuardMessageStorageThreadStructureEnhanced(session, (ISimplifiedThreadStructureEnhanced) instance, pgpMailRecognizer, authToken);
        }

        if (ISimplifiedThreadStructure.class == iface) {
            return (T) new GuardMessageStorageThreadStructure(session, (ISimplifiedThreadStructure) instance, pgpMailRecognizer, authToken);
        }

        return instance;
    }

    protected GuardAuthenticationToken getAuthToken(Session session) throws OXException {
        return this.authToken != null ?
            this.authToken :
            new AuthenticationTokenHandler().getForSession(session);
    }

    /**
     * Checks whether the given message is a PGP message
     *
     * @param message
     * @return True, if the message is a PGP message, false otherwise
     * @throws OXException
     */
    private boolean isPGPMessage(MailMessage message) throws OXException {
        return pgpMailRecognizer.isPGPMessage(message);
    }

    /**
     * Checks if plain-text message is signed
     *
     * @param message
     * @return
     * @throws OXException
     */
    private boolean isPGPSignedMessage(MailMessage message) throws OXException {
        return pgpMailRecognizer.isPGPSignedMessage(message);
    }

    /**
     * Internal method to process a MimeMessage using OX Guard
     *
     * @param authToken The authToken, might be null for messages which does not require an authentication (for example verifying signed messages)
     * @param message The message
     * @return The processed (i.e decrypted and/or verified) message
     * @throws OXException
     */
    protected MailMessage processMessage(MailMessage message) throws OXException {
        GuardAuthenticationToken authToken = getAuthToken(session);
        final GuardMimeDecrypter decrypter = new GuardMimeDecrypter();

        message.removeHeader("X-Guard-Signature-Result");  // Remove previous header if there

        // Copy flags
        int flags = message.containsFlags() ? message.getFlags() : -1;
        int label = message.containsColorLabel() ? message.getColorLabel() : -1;
        String[] userFlags = message.containsUserFlags() ? message.getUserFlags() : null;
        int threadLevel = message.containsThreadLevel() ? message.getThreadLevel() : -1;

        boolean changed = false;
        if (isPGPMessage(message)){
            if(authToken != null) {
                message = decrypter.decrypt(session, authToken, message);
                changed = true;
            }
            else {
                LOG.error("Missing / Bad authentication");
                throw GuardExceptionCodes.MISSING_AUTH.create();
            }
        }
        if (isPGPSignedMessage(message)) {
            GuardMimeVerifier verifier = new GuardMimeVerifier();
            message = verifier.verify(session, message);
            changed = true;
        }
        if (changed) {
            // Restore flags to decoded email
            if (flags >= 0) {
                message.setFlags(flags);
            }
            if (label >= 0) {
                message.setColorLabel(label);
            }
            if (null != userFlags) {
                message.addUserFlags(userFlags);
            }
            if (threadLevel >= 0) {
                message.setThreadLevel(threadLevel);
            }
        }

        return message;
    }


    /* (non-Javadoc)
     * @see com.openexchange.mail.api.IMailMessageStorage#updateMessageFlags(java.lang.String, java.lang.String[], int, java.lang.String[], boolean)
     */
    @Override
    public void updateMessageFlags(String folder, String[] mailIds, int flags, String[] userFlags, boolean set) throws OXException {
        delegate.updateMessageFlags(folder, mailIds, flags, userFlags, set);
    }

    /* (non-Javadoc)
     * @see com.openexchange.mail.api.MailMessageStorage#appendMessages(java.lang.String, com.openexchange.mail.dataobjects.MailMessage[])
     */
    @Override
    public String[] appendMessages(String destFolder, MailMessage[] msgs) throws OXException {
        return delegate.appendMessages(destFolder, msgs);
    }

    /* (non-Javadoc)
     * @see com.openexchange.mail.api.MailMessageStorage#copyMessages(java.lang.String, java.lang.String, java.lang.String[], boolean)
     */
    @Override
    public String[] copyMessages(String sourceFolder, String destFolder, String[] mailIds, boolean fast) throws OXException {
        return delegate.copyMessages(sourceFolder, destFolder, mailIds, fast);
    }

    @Override
    public void deleteMessages(String folder, String[] mailIds, boolean hardDelete) throws OXException {
        delegate.deleteMessages(folder, mailIds, hardDelete);
    }

    @Override
    public MailMessage[] getAllMessages(String folder, IndexRange indexRange, MailSortField sortField, OrderDirection order, MailField[] fields) throws OXException {
        return delegate.getAllMessages(folder, indexRange, sortField, order, fields);
    }

    @Override
    public String[] getPrimaryContents(String folder, String[] mailIds) throws OXException {
        return delegate.getPrimaryContents(folder, mailIds);
    }

    @Override
    public MailMessage[] getMessages(String folder, String[] mailIds, MailField[] fields) throws OXException {
        MailMessage[] messages = delegate.getMessages(folder, mailIds, fields);
        if (null == messages || messages.length <= 0) {
            return messages;
        }

        // Only decrypt if body should be fetched
        MailFields mailFields = new MailFields(fields);
        if (mailFields.contains(MailField.BODY) || mailFields.contains(MailField.FULL)) {
            for (int i = messages.length; i-- > 0;) {
                messages[i] = processMessage(messages[i]);
            }
        }

        return messages;
    }

    @Override
    public MailMessage[] getThreadSortedMessages(String folder, IndexRange indexRange, MailSortField sortField, OrderDirection order, SearchTerm<?> searchTerm, MailField[] fields) throws OXException {
        MailMessage[] messages = super.getThreadSortedMessages(folder, indexRange, sortField, order, searchTerm, fields);
        if (null == messages || messages.length <= 0) {
            return messages;
        }

        //Only decrypt if body should be fetched
        MailFields mailFields = new MailFields(fields);
        if (mailFields.contains(MailField.BODY) || mailFields.contains(MailField.FULL)) {
            final GuardAuthenticationToken authToken = new AuthenticationTokenHandler().getForSession(session);
            for (int i = messages.length; i-- > 0;) {
                messages[i] = processMessage(messages[i]);
            }
        }

        return messages;
    }

    @Override
    public MailMessage getMessage(String folder, String mailId, boolean markSeen) throws OXException {
        MailMessage message = delegate.getMessage(folder, mailId, markSeen);
        if (null == message) {
            return message;
        }

        return processMessage(message);
    }

    @Override
    public MailMessage[] getUnreadMessages(String folder, MailSortField sortField, OrderDirection order, MailField[] fields, int limit) throws OXException {
        return delegate.getUnreadMessages(folder, sortField, order, fields, limit);
    }

    @Override
    public String[] moveMessages(String sourceFolder, String destFolder, String[] mailIds, boolean fast) throws OXException {
        return delegate.moveMessages(sourceFolder, destFolder, mailIds, fast);
    }

    @Override
    public void releaseResources() throws OXException {
        delegate.releaseResources();
    }

    @Override
    public MailMessage saveDraft(String draftFullname, ComposedMailMessage draftMail) throws OXException {
        //TODO
        return delegate.saveDraft(draftFullname, draftMail);
    }

    /* (non-Javadoc)
     * @see com.openexchange.mail.api.MailMessageStorage#searchMessages(java.lang.String, com.openexchange.mail.IndexRange, com.openexchange.mail.MailSortField, com.openexchange.mail.OrderDirection, com.openexchange.mail.search.SearchTerm, com.openexchange.mail.MailField[])
     */
    @Override
    public MailMessage[] searchMessages(String folder, IndexRange indexRange, MailSortField sortField, OrderDirection order, SearchTerm<?> searchTerm, MailField[] fields) throws OXException {
        MailMessage[] messages = delegate.searchMessages(folder, indexRange, sortField, order, searchTerm, fields);

        if (null == messages || messages.length <= 0) {
            return messages;
        }

        //Only decrypt if body should be fetched
        MailFields mailFields = new MailFields(fields);
        if (mailFields.contains(MailField.BODY) || mailFields.contains(MailField.FULL)) {
            final GuardAuthenticationToken authToken = new AuthenticationTokenHandler().getForSession(session);
            for (int i = messages.length; i-- > 0;) {
                messages[i] = processMessage(messages[i]);
            }
        }

        return messages;
    }

    @Override
    public void updateMessageColorLabel(String folder, String[] mailIds, int colorLabel) throws OXException {
        delegate.updateMessageColorLabel(folder, mailIds, colorLabel);
    }

    @Override
    public void updateMessageUserFlags(String folder, String[] mailIds, String[] flags, boolean set) throws OXException {
        delegate.updateMessageUserFlags(folder, mailIds, flags, set);
    }

    @Override
    public void updateMessageFlags(String folder, String[] mailIds, int flags, boolean set) throws OXException {
        delegate.updateMessageFlags(folder, mailIds, flags, set);
    }

    @Override
    public MailMessage[] getNewAndModifiedMessages(String folder, MailField[] fields) throws OXException {
        return delegate.getNewAndModifiedMessages(folder, fields);
    }

    @Override
    public MailMessage[] getDeletedMessages(String folder, MailField[] fields) throws OXException {
        return delegate.getDeletedMessages(folder, fields);
    }

    @Override
    public int getUnreadCount(String folder, SearchTerm<?> searchTerm) throws OXException {
        return delegate.getUnreadCount(folder, searchTerm);
    }

    /* (non-Javadoc)
     * @see com.openexchange.mail.api.IMailMessageStorageDelegator#getDelegateMessageStorage()
     */
    @Override
    public IMailMessageStorage getDelegateMessageStorage() throws OXException {
        return delegate;
    }
}
