/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.mail.service.impl;

import java.util.ArrayList;
import java.util.List;
import com.openexchange.exception.OXException;
import com.openexchange.guard.internal.authentication.AuthenticationTokenHandler;
import com.openexchange.guard.internal.authentication.GuardAuthenticationToken;
import com.openexchange.mail.MailField;
import com.openexchange.mail.MailFields;
import com.openexchange.mail.MailSortField;
import com.openexchange.mail.OrderDirection;
import com.openexchange.mail.api.IMailMessageStorage;
import com.openexchange.mail.api.IMailMessageStorageThreadReferences;
import com.openexchange.mail.dataobjects.MailMessage;
import com.openexchange.mail.dataobjects.MailThread;
import com.openexchange.mail.mime.crypto.PGPMailRecognizer;
import com.openexchange.mail.search.SearchTerm;
import com.openexchange.session.Session;


/**
 * {@link GuardMessageStorageThreadReferences} - The Guard-aware message storage delegating to {@link IMailMessageStorageThreadReferences}.
 *
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 * @since v7.8.4
 */
public class GuardMessageStorageThreadReferences extends GuardMessageStorage implements IMailMessageStorageThreadReferences {

    private final IMailMessageStorageThreadReferences messageStorageThreadReferences;

    /**
     * Initializes a new {@link GuardMessageStorageThreadReferences}.
     *
     * @param session A {@link Session} a session object
     * @param delegate The underlying {@link IMailMessageStorage} to wrap with guard.
     * @param pgpMailRecognizer A {@link PGPMailRecognizer} object recognizes PGP messages.
     * @param authToken
     */
    public GuardMessageStorageThreadReferences(Session session, IMailMessageStorageThreadReferences delegate, PGPMailRecognizer pgpMailRecognizer, GuardAuthenticationToken authToken) {
        super(session, delegate, pgpMailRecognizer, authToken);
        this.messageStorageThreadReferences = delegate;
    }

    @Override
    public boolean isThreadReferencesSupported() throws OXException {
        return messageStorageThreadReferences.isThreadReferencesSupported();
    }

    @Override
    public List<MailThread> getThreadReferences(String fullName, int size, MailSortField sortField, OrderDirection order, SearchTerm<?> searchTerm, MailField[] fields, String[] headerNames) throws OXException {
        List<MailThread> threads = messageStorageThreadReferences.getThreadReferences(fullName, size, sortField, order, searchTerm, fields, headerNames);
        if (null == threads || threads.isEmpty()) {
            return threads;
        }

        // Only decrypt if body should be fetched
        MailFields mailFields = new MailFields(fields);
        if (mailFields.contains(MailField.BODY) || mailFields.contains(MailField.FULL)) {
            GuardAuthenticationToken authToken = new AuthenticationTokenHandler().getForSession(session);
            List<MailThread> processedThreads = new ArrayList<>(threads.size());
            for (MailThread mailThread : threads) {
                processedThreads.add(processMailThread(authToken, mailThread));
            }
            threads = processedThreads;
        }

        return threads;
    }

    private MailThread processMailThread(GuardAuthenticationToken authToken, MailThread mailThread) throws OXException {
        MailThread processed;
        {
            MailMessage parent = mailThread.getParent();
            if (null == parent) {
                processed = new MailThread();
            } else {
                processed = new MailThread(processMessage(parent));
            }
        }

        List<MailThread> children = mailThread.getChildren();
        List<MailThread> processedChildren = new ArrayList<>(children.size());
        for (MailThread child : children) {
            processedChildren.add(processMailThread(authToken, child));
        }
        processed.addChildren(processedChildren);
        return processed;
    }

}
